/* Copyright (C) 2003 MySQL AB

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <mysql.h>
#include <ndbapi/NdbApi.hpp>
#include <mgmapi.h>
#include <mgmapi_debug.h>
#include <stdio.h>
#include <time.h>
#include <signal.h>
#include <errno.h>
#include <unistd.h>
#include <libgen.h>
#include <map>

#define MGMERROR(h) \
{ \
fprintf(stderr, "code: %d msg: %s\n", \
  ndb_mgm_get_latest_error(h), \
  ndb_mgm_get_latest_error_msg(h)); \
exit(-1); \
}

#define LOGEVENTERROR(h) \
{ \
fprintf(stderr, "code: %d msg: %s\n", \
  ndb_logevent_get_latest_error(h), \
  ndb_logevent_get_latest_error_msg(h)); \
exit(-1); \
}
using namespace std;

int main(int argc, char** argv)
{
  NdbMgmHandle h;
  NdbLogEventHandle le;
  int filter[] = { 15, NDB_MGM_EVENT_CATEGORY_BACKUP, 0 };

  struct ndb_logevent event;

  if (argc < 2)
  {
    printf("Arguments are <connect_string>\n");
    exit(-1);
  }
  const char *connectstring = argv[1];

  //ndb_init();

  h= ndb_mgm_create_handle();
  if ( h == 0)
  {
    printf("Unable to create handle\n");
    exit(-1);
  }
  if (ndb_mgm_set_connectstring(h, connectstring) == -1)
  {
    printf("Unable to set connectstring\n");
    exit(-1);
  }

  ndb_mgm_set_timeout(h,4000);
  if (ndb_mgm_connect(h,0,0,0)) MGMERROR(h);

  le= ndb_mgm_create_logevent_handle(h, filter);
  if ( le == 0 )  MGMERROR(h);

  
  struct ndb_mgm_reply reply;
  unsigned int backup_id;
  printf("starting backup\n"); 
  ndb_mgm_start_backup(h,1,&backup_id,&reply);
  printf("backup started: %d\n",backup_id);

  while (1)
  {
    int timeout= 5000;
    int r= ndb_logevent_get_next(le,&event,timeout);

    if (r < 0) {
      LOGEVENTERROR(le);
    } else {


      switch (event.type) {
      case NDB_LE_BackupStarted: 
        printf("Backup %d started event on %d\n",event.BackupStarted.backup_id,event.BackupStarted.starting_node);
	break;
      case NDB_LE_BackupCompleted:
        printf("Backup %d finished event on %d\n",event.BackupCompleted.backup_id,event.BackupCompleted.starting_node);
        printf("start_gci: %d stop_gci: %d\n",event.BackupCompleted.start_gci,event.BackupCompleted.stop_gci);
        break;
      default:
	break;
      }
    }
  }
  ndb_mgm_destroy_logevent_handle(&le);
  ndb_mgm_destroy_handle(&h);
  ndb_end(0);
  return 0;
}
