#!/usr/bin/perl

#
# config entries
#

$ndb_dir="/var/lib/mysql-cluster";
$NDB_MGM="/usr/bin/ndb_mgm";

$TAG="mysql_cluster";
$CUSTOMER="CUSTOMER";
$SERVICE="SERVICE";
$PRIO="user.notice";

$VAL_FATAL=95;
$VAL_CRIT=90;
$VAL_WARN=80;

$SC_FATAL="FATAL";
$SC_CRIT="CRITICAL";
$SC_WARN="WARNING";
$SC_INFO="INFO";
$SC_OK="OK";
$SC_DEBUG="DEBUG";

sub get_status { 

  my $maxdata=shift || 0;
  my $maxindex=shift || 0;
  my $exitcode=0;
  my $SC=$SC_OK;

  if (($maxdata >= $VAL_WARN) || ($maxindex >= $VAL_WARN)) { 
    $SC=$SC_WARN;
    $exitcode=1;
  } 
  # Don't use else here - we want highest of either to set the value.
  if (($maxdata >= $VAL_CRIT) || ($maxindex >= $VAL_CRIT)) {
    $SC=$SC_CRITICAL;
    $exitcode=2; 
  }
  if ( ($runstyle eq "syslog") && ( $maxdata >= $VAL_FATAL ) ){
    $SC=$SC_FATAL;
    $exitcode=2;
  }
   
  return wantarray ?  ($SC,$exitcode) : $SC;
}
# no more config entries below

$runstyle = shift || "syslog";
$want_nodeid = shift || 0; 

if ($runstyle eq "syslog" ) { 
  use Sys::Syslog; 
  openlog($TAG,'pid,cons','user');
}
# configuration checks
if ( ! -r glob("$ndb_dir/ndb_*_cluster.log") ) {
  print "Can't read cluster log in $ndb_dir\n";
  exit 3 ;
}
if ( ! -x "$NDB_MGM" ) {
  print "Can't execute management client $NDB_MGM\n";
  exit 3 ;
}

#
# monitor cluster log while script is running
#

# we rely on only one cluster.log file being there matching 
# the wildcard, so we don't need to configure the node id
open(LOG,glob("$ndb_dir/ndb_*_cluster.log")) || exit 3;
seek(LOG,0,2); # seek to end of file


## request nodes to report their memory usage,
## the results are written to the cluster log
`$NDB_MGM -e 'all dump 1000'`;

my $maxdata = 0;
my $maxindex= 0;

foreach (<LOG>) {

#INFO     -- Node 1: Data usage is 26%(12956 32K pages of total 48000)

  /usage/ || next;

  s/.*?\[MgmSrvr\]//;
  my $line=$_;
  chomp($line);
  s/.*?Node\s*(\d+):\s*(Data|Index).*?(\d+)\%\((\d+).*?(\d+)\).*/$1,$2,$3,$4,$5/sg;
  my ($nodeid,$data,$percent,$usage,$topusage) = split(",");
  if (($nodeid == 0) || ($want_nodeid == $nodeid)) { 
    if ($runstyle eq "cacti") {
      print "Node$nodeid" unless ($nodeid > 0);
       
      print("$data:$usage Percent$data:$percent ");
    }
    if ($data eq "Index") { 
      $maxindex = ($maxindex > $percent) ? $maxindex : $percent; 
      $indexusage=$usage; $indextop=$topusage;
    } elsif ($data eq "Data") { 
      $maxdata = ($maxdata > $percent) ? $maxdata : $percent; 
      $datausage=$usage; $datatop=$topusage;
    } 
    if ( $runstyle eq "syslog") {
      $SC=get_status($data);
      syslog($PRIO,"$CUSTOMER $SERVICE $SC $line");
    }
  }
}



if ($runstyle eq "nagios") {
  ($SC,$exitcode) = get_status($maxdata,$maxindex);
  print "$SC - ndb: Data - $maxdata% ($datausage/$datatop), Index - $maxindex% ($indexusage/$indextop)|data=$maxdata%;$VAL_CRIT;$VAL_WARN;0; index=$maxindex%;$VAL_CRIT;$VAL_WARN;0;\n";
}

if ($runstyle eq "syslog") {
  closelog();
}

exit $exitcode;
