/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
 *  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
 *
 * Copyright (C) 2008 Sun Microsystems

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; version 2 of the License.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <mysql.h>
#include <ndbapi/NdbApi.hpp>
#include <mgmapi.h>
#include <mgmapi_debug.h>
#include <stdio.h>
#include <time.h>
#include <signal.h>
#include <errno.h>
#include <unistd.h>
#include <libgen.h>
#include <map>

#include <popt.h>




#define MGMERROR(h)                             \
  {                                                     \
    fprintf(stderr, "code: %d msg: %s\n",               \
            ndb_mgm_get_latest_error(h),                \
            ndb_mgm_get_latest_error_msg(h));           \
    exit(-1);                                           \
  }


#define LOGEVENTERROR(h)                                \
  {                                                     \
    fprintf(stderr, "code: %d msg: %s\n",               \
            ndb_logevent_get_latest_error(h),           \
            ndb_logevent_get_latest_error_msg(h));      \
  }
//exit(-1);                                                             \
/** Log event specific data for for corresponding NDB_LE_ log event */
typedef struct {
  unsigned trans_count;
  unsigned commit_count;
  unsigned read_count;
  unsigned simple_read_count;
  unsigned write_count;
  unsigned attrinfo_count;
  unsigned conc_op_count;
  unsigned abort_count;
  unsigned scan_count;
  unsigned range_scan_count;
} TransactionCt;
/** Log event specific data for for corresponding NDB_LE_ log event */
typedef struct {
  unsigned ops;
} OperationCt;

typedef struct {
  unsigned page_size;
  unsigned pages_used;
  unsigned pages_total;
  unsigned block;
} MemoryUsage;

#define MAXNODES 64

using namespace std;

int main(int argc, const char** argv)
{

  NdbMgmHandle h;
  NdbLogEventHandle le;
  int filter[] = { 15, NDB_MGM_EVENT_CATEGORY_STATISTIC, 0 };

  MemoryUsage node_datamem[MAXNODES];
  MemoryUsage node_indexmem[MAXNODES];
  MemoryUsage total_datamem;
  MemoryUsage total_indexmem;

  //initialize arrays
  for (Uint32 x = 0; x<MAXNODES;x++){
    node_datamem[x].page_size=32768;
    node_datamem[x].pages_used=0;
    node_datamem[x].pages_total=0;
    node_datamem[x].block=0;
    node_indexmem[x].page_size=8192;
    node_indexmem[x].pages_used=0;
    node_indexmem[x].pages_total=0;
    node_indexmem[x].block=0;
  }
  total_datamem.page_size=32768;
  total_datamem.pages_used=0;
  total_datamem.pages_total=0;
  total_datamem.block=0;
  total_indexmem.page_size=8192;
  total_indexmem.pages_used=0;
  total_indexmem.pages_total=0;
  total_indexmem.block=0;


  static char * fileName;
  static char * connectString;
  bool verbose = 0;

  char *tmpfilename= (char*)malloc(PATH_MAX);
  connectString = (char*)malloc(PATH_MAX);
  fileName= (char*)malloc(PATH_MAX);
  strncpy(connectString, "localhost", PATH_MAX);
  strncpy(fileName, "/var/lib/mysql-cluster/ndb_status", PATH_MAX);


  const struct poptOption options[] = {
    {"file",'f',POPT_ARG_STRING,fileName,'f',
     "filename to write to","a file name"},
    {"verbose",'v',POPT_ARG_NONE,&verbose,'v',
     "be verbose",NULL},
    {"connectstring",'c',POPT_ARG_STRING, (void *)connectString,'c',
     "cluster connect string","a connection string"},
    POPT_AUTOHELP
    POPT_TABLEEND
  };


  struct ndb_logevent event;

  poptContext context = poptGetContext(NULL, argc, argv,
                                       options, 0);

  int rc;
  bool write_to_file = false;
  while ((rc = poptGetNextOpt(context)) > 0) {
    const char * theArg = poptGetOptArg(context);
    switch (rc) {
    case 'f':
      strncpy(fileName, theArg, PATH_MAX);
      write_to_file = true;
      break;
    case 'c':
      strncpy(connectString, theArg, PATH_MAX);
      break;
    }
  }
  poptFreeContext(context);

  strncpy(tmpfilename, fileName, PATH_MAX);

  if (write_to_file) {
    if (((access(fileName,F_OK) == 0) && (access (fileName, W_OK) == -1)) ||
        (access(fileName,F_OK) == -1) &&
        (access(dirname(tmpfilename),W_OK)==-1))
    {
      printf("Can't write to %s - aborting\n",fileName);
      return 1;
    }

  }
  ndb_init();

  TransactionCt trans_count;
  OperationCt op_count;

  h= ndb_mgm_create_handle();
  if ( h == 0)
  {
    printf("Unable to create handle\n");
    exit(-1);
  }
  if (ndb_mgm_set_connectstring(h, connectString) == -1)
  {
    printf("Unable to set connectstring\n");
    exit(-1);
  }

  ndb_mgm_set_timeout(h,4000);
  if (ndb_mgm_connect(h,0,0,0)) MGMERROR(h);

  le= ndb_mgm_create_logevent_handle(h, filter);
  if ( le == 0 )  MGMERROR(h);


  trans_count.trans_count=0;
  trans_count.commit_count=0;
  trans_count.read_count=0;
  trans_count.simple_read_count=0;
  trans_count.write_count=0;
  trans_count.attrinfo_count=0;
  trans_count.conc_op_count=0;
  trans_count.abort_count=0;
  trans_count.scan_count=0;
  trans_count.range_scan_count=0;
  op_count.ops=0;


  map<int,bool> nodes_present;

  time_t the_time=(time_t)0;
  time_t the_old_time=(time_t)0;
  time_t the_old_dump_time=(time_t)0;

  time(&the_time);
  the_old_dump_time = the_time - (time_t)(300-10);

  while (1)
  {
    int timeout= 5000;
    int r= ndb_logevent_get_next(le,&event,timeout);

    if (r < 0) {
      LOGEVENTERROR(le);
    } else {


      switch (event.type) {
      case NDB_LE_TransReportCounters:
        if (verbose) {
          printf("%d trans:%d read:%d write:%d commit:%d abort:%d "
                 "range:%d scan:%d pk:%d \n",
                 event.source_nodeid,
                 event.TransReportCounters.trans_count,
                 event.TransReportCounters.read_count,
                 event.TransReportCounters.write_count,
                 event.TransReportCounters.commit_count,
                 event.TransReportCounters.abort_count,
                 event.TransReportCounters.range_scan_count,
                 event.TransReportCounters.scan_count,
                 event.TransReportCounters.simple_read_count
                 );
        }
        trans_count.trans_count+=event.TransReportCounters.trans_count;
        trans_count.commit_count+=event.TransReportCounters.commit_count;
        trans_count.read_count+=event.TransReportCounters.read_count;
        trans_count.simple_read_count+=
          event.TransReportCounters.simple_read_count;
        trans_count.write_count+=
          event.TransReportCounters.write_count;
        trans_count.attrinfo_count+=
          event.TransReportCounters.attrinfo_count;
        trans_count.conc_op_count+=
          event.TransReportCounters.conc_op_count;
        trans_count.abort_count+=event.TransReportCounters.abort_count;
        trans_count.scan_count+=event.TransReportCounters.scan_count;
        trans_count.range_scan_count+=
          event.TransReportCounters.range_scan_count;

        nodes_present[event.source_nodeid]=true;
        break;
      case NDB_LE_OperationReportCounters:
        op_count.ops+=event.OperationReportCounters.ops;
        break;

      case NDB_LE_MemoryUsage:
        MemoryUsage* theMemoryUsage;
        switch (event.MemoryUsage.page_size_kb) {
        case 8192:
          theMemoryUsage=&(node_indexmem[event.source_nodeid]);
          break;
        case 32768:
          theMemoryUsage=&(node_datamem[event.source_nodeid]);
          break;
        default:
          break;
        }
        theMemoryUsage->pages_used=event.MemoryUsage.pages_used;
        theMemoryUsage->pages_total=event.MemoryUsage.pages_total;
        theMemoryUsage->block=event.MemoryUsage.block;

        /*
          printf("NodeId:%d PageSize:%d PagesUsed:%d PagesTotal:%d "
          "Blocks:%d\n",
          event.source_nodeid,
          event.MemoryUsage.page_size_kb,
          theMemoryUsage->pages_used,
          theMemoryUsage->pages_total,
          theMemoryUsage->block);
        */

        break;
      default:
        break;
      }
    }

    total_datamem.pages_used=0;
    total_datamem.pages_total=0;
    total_datamem.block=0;
    total_indexmem.pages_used=0;
    total_indexmem.pages_total=0;
    total_indexmem.block=0;


    for (map<int,bool>::iterator iter = nodes_present.begin() ;
         iter != nodes_present.end() ;
         iter++){
      total_datamem.pages_used += node_datamem[iter->first].pages_used;
      total_datamem.pages_total += node_datamem[iter->first].pages_total;
      total_datamem.block += node_datamem[iter->first].block;
      total_indexmem.pages_used += node_indexmem[iter->first].pages_used;
      total_indexmem.pages_total += node_indexmem[iter->first].pages_total;
      total_indexmem.block += node_indexmem[iter->first].block;
    }

    time(&the_time);
    if (the_time > the_old_time+(time_t)5) {
      if (write_to_file) {
        FILE* fd = fopen(fileName,"w");
        if(!fd)
        {
          perror(strerror(errno));
          exit(-1);
        }

        //printf("%d,", (unsigned int)the_time);
        fprintf(fd,"Trans_count:%d ", trans_count.trans_count);
        fprintf(fd,"Read_count:%d ", trans_count.read_count);
        fprintf(fd,"Commit_count:%d ", trans_count.commit_count);
        fprintf(fd,"Scan_count:%d ", trans_count.scan_count);
        fprintf(fd,"Simple_read_count:%d ", trans_count.simple_read_count);
        fprintf(fd,"Write_count:%d ",trans_count.write_count);
        fprintf(fd,"Attrinfo_count:%d ",trans_count.attrinfo_count);
        fprintf(fd,"Conc_op_count:%d ",trans_count.conc_op_count);
        fprintf(fd,"Abort_count:%d ",trans_count.abort_count);
        fprintf(fd,"Range_scan_count:%d ",trans_count.range_scan_count);
        fprintf(fd,"Operations:%d ", op_count.ops);
        fprintf(fd,"TotalDataMemoryUsed:%d ",
                total_datamem.pages_used*total_datamem.page_size);
        fprintf(fd,"TotalDataMemoryTotal:%d ",
                total_datamem.pages_total*total_datamem.page_size);
        fprintf(fd,"TotalDataPagesUsed:%d ",total_datamem.pages_used);
        fprintf(fd,"TotalDataPagesTotal:%d ",total_datamem.pages_total);
        fprintf(fd,"TotalDataBlocks:%d ",total_datamem.block);
        fprintf(fd,"TotalIndexMemoryUsed:%d ",
                total_indexmem.pages_used*total_indexmem.page_size);
        fprintf(fd,"TotalIndexMemoryTotal:%d ",
                total_indexmem.pages_total*total_indexmem.page_size);
        fprintf(fd,"TotalIndexPagesUsed:%d ",total_indexmem.pages_used);
        fprintf(fd,"TotalIndexPagesTotal:%d ",total_indexmem.pages_total);
        fprintf(fd,"TotalIndexBlocks:%d ",total_indexmem.block);

        for (map<int,bool>::iterator iter = nodes_present.begin() ;
             iter != nodes_present.end() ;
             iter++){
          fprintf(fd,"DataMemoryUsed%d:%d ", iter->first,
                  node_datamem[iter->first].pages_used*
                  node_datamem[iter->first].page_size);
          fprintf(fd,"DataMemoryTotal%d:%d ", iter->first,
                  node_datamem[iter->first].pages_total*
                  node_datamem[iter->first].page_size);
          fprintf(fd,"DataPagesUsed%d:%d ", iter->first,
                  node_datamem[iter->first].pages_used);
          fprintf(fd,"DataPagesTotal%d:%d ", iter->first,
                  node_datamem[iter->first].pages_total);
          fprintf(fd,"DataBlocks%d:%d ", iter->first,
                  node_datamem[iter->first].block);
          fprintf(fd,"IndexMemoryUsed%d:%d ", iter->first,
                  node_indexmem[iter->first].pages_used*
                  node_indexmem[iter->first].page_size);
          fprintf(fd,"IndexMemoryTotal%d:%d ", iter->first,
                  node_indexmem[iter->first].pages_total*
                  node_indexmem[iter->first].page_size);
          fprintf(fd,"IndexPagesUsed%d:%d ", iter->first,
                  node_indexmem[iter->first].pages_used);
          fprintf(fd,"IndexPagesTotal%d:%d ", iter->first,
                  node_indexmem[iter->first].pages_total);
          fprintf(fd,"IndexBlocks%d:%d ", iter->first,
                  node_indexmem[iter->first].block);
        }
        fprintf(fd,"\n");
        fclose(fd);

      }
      /*
        printf("TotalDataPagesTotal:%d ",total_datamem.pages_total);
        printf("IndexDataPagesUsed:%d ",total_indexmem.pages_used);
        printf("IndexDataBlocks:%d ",total_indexmem.block);
        printf("TotalDataPagesTotal:%d ",total_datamem.pages_total);
        printf("IndexDataPagesUsed:%d ",total_indexmem.pages_used);
        printf("IndexDataBlocks:%d \n",total_indexmem.block);
      */

      the_old_time = the_time;
    }

    if (the_time > the_old_dump_time+(time_t)300) {
      for (map<int,bool>::iterator iter = nodes_present.begin() ;
           iter != nodes_present.end() ;
           iter++){

        int pars[1];
        pars[0]=1000;
        struct ndb_mgm_reply reply;

        int ret = ndb_mgm_dump_state(h, iter->first, pars, 1, &reply);
        if (ret == -1) {
          printf("* %5d: %s",
                 ndb_mgm_get_latest_error(h),
                 ndb_mgm_get_latest_error_msg(h));
          printf("*        %s\n", ndb_mgm_get_latest_error_desc(h));
          exit(-1);
        }
      }

      the_old_dump_time = the_time;
    }

  }

  ndb_mgm_destroy_logevent_handle(&le);
  ndb_mgm_destroy_handle(&h);
  ndb_end(0);
  return 0;
}

